/**
 * @file
 * Javascript for blajax module.
 */

/**
 * Drupal API initialisation.
 */
Drupal.behaviors.blajax = function(context) {
  // Check whether blocks with toolbars exist and if so,
  // initialize them.
  if (typeof Drupal.settings.blajax == "object" && Drupal.settings.blajax.js) {
    Drupal.Blajax.setToolbars(context);
    Drupal.Blajax.setAutoRefresh(context);
    Drupal.Blajax.autoAttach(context);
  }
}

// Create our own namespace and initialize some default values;
Drupal.Blajax = {
  "Opacity": 1,
  "blockTimers": new Array
}

/**
 * Attach dynamic behaviors to the toolbar elements.
 */
Drupal.Blajax.autoAttach = function(context) {
  // Define dynamic actions for all toolbar links/icons.
  $('a.blajax-ajax-link:not(.blajax-link-processed)',context).click(function() {
    var a = this;
    var theBlock = $(a).parents('div.block');
    var operation = $(a).attr("class").match(/\bblajax-action-([^ ]+)/);
    operation = operation[1];
    var success = false;

    // Replace the clicked icon with a nice progress indicator when being
    // clicked (or, for non-CSS browsers, replace the link label with "...")
    // to present an immediate visible response.
    // (See blajax.css)
    $(a).css("color","transparent").html('<span class="blajax-ahah-progress" style="float: none"><span style="visibility: hidden">...</span></span>');

    href = $(this).attr('href');

    // Request the block action to be performed.
    // Hiding a block receives a success message ("data.warn")
    // if the user hides a block for the first time.
    // All other cases hide the block and initiate a second
    // Ajax request ("loadContent") that delivers updated block content.
    $.ajax({
      type: 'POST',
      data: { js: 1 },
      dataType: "json",
      url: href,
      global: true,
      destination: Drupal.settings.blajax.destination,
      success: function(data) {
        if (typeof data == "object" && data.success) {
          if (data.warn) {
            alert(data.warn);
          }
          switch(operation) {
            case "hide":
              if (Drupal.settings.blajax.animate) {
                theBlock.slideUp().remove();
              } else {
                theBlock.remove();
              }
              break;
            case "minimize":
              Drupal.Blajax.clearBlockTimer(Drupal.Blajax.getBlockID(theBlock));
            case "maximize":
            case "refresh":
              if (Drupal.settings.blajax.animate) {
                theBlock.slideUp();
              }
              Drupal.Blajax.loadContent(theBlock,"all");
              break;
          }
        } else {
          self.location.href = href;
        }
      },
      error: function(request,data,exception) {
        self.location.href = href;
      }
    });
    // Prevent the fall back REST method to be initiated.
    return false;
  }).addClass("blajax-link-processed");
}

/**
 * Add the configured visual properties to each toolbar.
 */
Drupal.Blajax.setToolbars = function(context) {
  // If toolbar position is set to top, move it there.
  $('div.blajax-toolbar.blajax-top:not(.blajax-processed-top)',context).each(function(){
    target = $(this).parents('div.block');
    $(this).remove().prependTo(target);

    // Swap margins (distance to block content).
    newTopMargin = $(this).css("marginBottom");
    newBottomMargin = $(this).css("marginTop");
    $(this).css("marginTop",newTopMargin).css("marginBottom",newBottomMargin)
      .addClass("blajax-processed-top");
  });

  // Set opacity.
  if (!Drupal.settings.blajax.opacity) {
    Drupal.Blajax.Opacity = 0;
  } else {
    Drupal.Blajax.Opacity = Drupal.settings.blajax.opacity/100;
  }
  // Configured opacity only applies as a default.
  // Whenever a toolbar is hovered, it is faded into 100% opacity
  // and back to the defined opacity on hover end.
  $('div.blajax-toolbar div.blajax-links:not(.blajax-processed-opaque)',context)
    .addClass("blajax-processed-opaque")
    .fadeTo(5,Drupal.Blajax.Opacity).parents('div.blajax-toolbar').hover(
    function() {$(this).children('div.blajax-links').fadeTo(5,1);},
    function() {$(this).children('div.blajax-links').fadeTo(5,Drupal.Blajax.Opacity);}
  );

  // Set block hover behavior.
  // Defines if/how to show/hide a toolbar when the corresponding block
  // is being entered or left by the mouse.
  switch (Drupal.settings.blajax.toolbarFX) {
    // Slide in/out:
    case '1':
      $('div.blajax-toolbar div.blajax-links:not(.blajax-processed-slide)',context)
        .addClass("blajax-processed-slide").hide().parents('div.block').hover(
        function() {$(this).find('div.blajax-links').slideDown(50);},
        function() {$(this).find('div.blajax-links').slideUp(50);}
      );
    break;
    // Fade in/out:
    case '2':
      $('div.blajax-toolbar div.blajax-links:not(.blajax-processed-overlay)',context)
        .addClass("blajax-processed-overlay").fadeTo(1,0).parents('div.block').hover(
        function() {$(this).find('div.blajax-links').fadeTo(5,Drupal.Blajax.Opacity);},
        function() {$(this).find('div.blajax-links').fadeTo(1,0);}
      );
    break;
  }
}

/**
 * Load refreshed block content.
 *
 * Applies whenever a block has been restored (maximized),
 * minimized or (auto) refreshed. Retrieves block caption
 * (and contents, if applying) and shows it.
 *
 * @param target
 *   The the target block's DOM container object.
 * @param scope
 *   Whether to receive the caption (subject) or both
 *   caption and block content (all).
 */
Drupal.Blajax.loadContent = function(target,scope) {
  if (!scope) {scope = null};

  // Extract module and delta from blockId:
  var bid = Drupal.Blajax.getBlockID(target);
  href = Drupal.settings.basePath+'js/blajax/get_block';
  var moduleName = Drupal.Blajax.getBlockProperty(bid,"module");
  var moduleDelta = Drupal.Blajax.getBlockProperty(bid,"delta");
  $.ajax({
    url: href,
    dataType: "json",
    type: 'POST',
    data: "js=1&module="+moduleName+"&delta="+moduleDelta+"&bid="+bid+"&scope="+scope+"&destination="+Drupal.settings.blajax.destination,
    success: function(data) {
      if (typeof data == "object" && data.content) {
        switch(scope) {
          case "subject":
            target.children('div.blajax-toolbar-caption:first')
              .html(data.content)
            ;
          break;
          case "all":
            target.children('div.content:first').replaceWith(data.content);
            newContent = target.children('div.block:first').html();
            if (newContent) {
              target.html(newContent);
              if (Drupal.settings.blajax.animate) {
                target.slideDown();
              } else {
                target.show();
              }
            }
            Drupal.attachBehaviors(target);
          break;
        }
      }
    },
    error: function(a,b,c) {
      // We could display an error message here.
      // But silence is preferred for now.
    }
  });
}

/**
 * Set autorefresh intervals for all applying blocks.
 *
 * Prepares the next automatic refreshment of each fully visible
 * block.
 */
Drupal.Blajax.setAutoRefresh = function(context) {
  $('div.blajax-autorefresh.blajax-maximized:not(.blajax-timer-processed)',context).each(
    function() {
      bid = Drupal.Blajax.getBlockID($(this).parents('div.block'));
      if (!Drupal.Blajax.blockTimers[bid]) {

        // Actually we do not create a real interval but
        // only a timeout that will be renewed with the scheduled
        // action. The timer ID for each block is stored in the
        // global blockTimers array so it can be deleted when
        // necessary, e.g. when a block is being minimized or
        // hidden in the meantime.
        Drupal.Blajax.blockTimers[bid] = window.setTimeout(
          "Drupal.Blajax.refreshBlock("+bid+")",
          Drupal.Blajax.getBlockProperty(bid,"autorefresh")*1000
        );
      }
    }
  ).addClass("blajax-timer-processed");
}

/**
 * Helper function; extracts a block ID from a CSS ID.
 *
 * @param theBlock
 *   DOM object representing the block in question.
 *
 * @return int
 *   The bare Drupal block ID.
 */
Drupal.Blajax.getBlockID = function(theBlock) {
  bid = theBlock.find('div.blajax-toolbar')
    .attr("id").match(/^blajax-toolbar-([0-9]+)$/)
  ;
  return bid[1];
}

/**
 * Retrieves a block property from the settings array.
 *
 * @param bid
 *   Drupal block ID (bid)
 * @param key
 *   The identifier of the property in question, e.g. "module"
 *
 * @return string|int|null
 *   On success: The corresponding value, otherwise null.
 */
Drupal.Blajax.getBlockProperty = function(bid,key) {
  var ret;
  eval("ret = Drupal.settings.blajax.blockProperties.b"+bid+"."+key);
  return ret;
}

/**
 * Rebuilds a block on autorefresh.
 *
 * @param bid
 *   The Drupal block ID of the block in question.
 */
Drupal.Blajax.refreshBlock = function(bid) {
  Drupal.Blajax.clearBlockTimer(bid);
  theBlock = $('div#blajax-toolbar-'+bid).parents('div.block');
  theBlock.find('div.blajax-timer-processed').removeClass("blajax-timer-processed");
  Drupal.Blajax.loadContent(theBlock,"all");
  Drupal.attachBehaviors(theBlock);
}

/**
 * Deletes an auto refresh interval for a specified block.
 *
 * @param bid
 *   The Drupal block ID of the block in question.
 */
Drupal.Blajax.clearBlockTimer = function(bid) {
  window.clearTimeout(Drupal.Blajax.blockTimers[bid]);
  Drupal.Blajax.blockTimers[bid] = false;
}

